<?php

namespace Drupal\acquia_contenthub\EventSubscriber\UnserializeContentField;

use Drupal\acquia_contenthub\Event\UnserializeCdfEntityFieldEvent;
use Drupal\Component\Uuid\Uuid;
use Drupal\Core\Entity\EntityInterface;
use Drupal\depcalc\DependencyStack;
use Psr\Log\LoggerInterface;

/**
 * Base class for unserializers that handle referenced entities.
 */
class FieldEntityReferenceBase {

  /**
   * Content hub logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * FieldEntityReferenceBase constructor.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   Content hub logger.
   */
  public function __construct(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * Get an entity from the dependency stack.
   *
   * @param string $uuid
   *   The uuid of the entity to be retrieved.
   * @param \Drupal\acquia_contenthub\Event\UnserializeCdfEntityFieldEvent $event
   *   The subscribed event.
   * @param string|null $entity_type
   *   Specify the entity type directly. Overrides field metadata, target
   *   attribute.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The retrieved entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Exception
   */
  protected function getEntity(string $uuid, UnserializeCdfEntityFieldEvent $event, string $entity_type = NULL): ?EntityInterface {
    if (is_null($entity_type)) {
      $entity_type = $event->getFieldMetadata()['target'] ?? '';
    }
    $stack = $event->getStack();
    if (Uuid::isValid($uuid)) {
      return $this->getEntityByUuid($entity_type, $uuid, $stack, $event->getFieldName());
    }
    return NULL;
  }

  /**
   * Returns an entity by its uuid from the stack or loads by uuid.
   *
   * @param string $entity_type
   *   The entity's type.
   * @param string $uuid
   *   The uuid of the entity to be retrieved.
   * @param \Drupal\depcalc\DependencyStack $stack
   *   The dependency stack used for lookup.
   * @param string $field_name
   *   The field name.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity or null.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getEntityByUuid(string $entity_type, string $uuid, DependencyStack $stack, string $field_name): ?EntityInterface {
    // Check stack first because uuids are referenced from origin, not local.
    if ($stack->hasDependency($uuid)) {
      return $stack->getDependency($uuid)->getEntity();
    }
    // Only fall back to local uuids as an absolute last resort.
    if (empty($entity_type)) {
      throw new \Exception(sprintf("The %s field does not specify a metadata target. This is likely due to an unresolved dependency export process. Please check your relationships.", $field_name));
    }
    $storage = \Drupal::entityTypeManager()->getStorage($entity_type);
    $entities = $storage->loadByProperties(['uuid' => $uuid]);
    return !empty($entities) ? reset($entities) : NULL;
  }

  /**
   * Log message if entity is missing and is not pruned.
   *
   * @param string $uuid
   *   Entity uuid.
   */
  protected function log(string $uuid): void {
    $this->logger->error(
      'Entity with {uuid} not found in DependencyStack or subscriber\'s database while unserializing field values.',
      [
        'uuid' => $uuid,
      ]
    );
  }

}
